import {
  deriveSharedSecret,
  encrypt,
  retrieveOrGenerateKeyPair,
} from "../utils/crypto";

const ENCRYPTED_URL_INPUT_ID = "encrypted-url";
const MESSAGE_INPUT_ID = "message";
const ENCRYPT_BUTTON_ID = "encrypt";
const REQUEST_PUBLIC_KEY = "requestPublicKey";
const TEMPLATE = `
<section>
  <details>
    <summary>How it Works</summary>
    <ul>
      <li>Enter the information you'd like to send to the person whom generated this link.</li>
      <li>Press "Encrypt" to generate a URL that has your encrypted information.</li>
      <li>Send this generated URL to the person who sent this information request.</li>
      <li>Only their browser can decrypt the secret in the URL.</li>
    </ul>
  </details>

  <form>
    <input
      type="text"
      name="message"
      id="${MESSAGE_INPUT_ID}"
      placeholder="Enter your message here..."
      aria-label="Message to Encrypt"
      required
    />
    <input
      id="${ENCRYPT_BUTTON_ID}"
      type="submit"
      value="Encrypt"
      aria-label="Encrypt"
    />
    <input
      type="text"
      name="encrypted-url"
      id="${ENCRYPTED_URL_INPUT_ID}"
      placeholder="Generated URL will appear here..."
      aria-label="Encrypted URL"
    />
  </form>
</section>
    `;

export async function setupSendPage(element: HTMLElement, key: string) {
  element.innerHTML = TEMPLATE;
  localStorage.setItem(REQUEST_PUBLIC_KEY, key);

  // Add an event listener to the "Encrypt" button
  const encryptButton = document.getElementById(
    ENCRYPT_BUTTON_ID
  ) as HTMLButtonElement;
  encryptButton.addEventListener("click", encryptData);
}

async function encryptData(event: Event) {
  event.preventDefault();
  const key = localStorage.getItem(REQUEST_PUBLIC_KEY)!;

  // Parse the 'p' parameter to get publicA
  const publicAJwk = JSON.parse(atob(key));

  // Import publicA as a CryptoKey
  const publicA = await window.crypto.subtle.importKey(
    "jwk",
    publicAJwk,
    {
      name: "ECDH",
      namedCurve: "P-256",
    },
    false,
    []
  );

  const keyPairB = await retrieveOrGenerateKeyPair();

  // Retrieve the message input
  const messageInput = document.getElementById(
    MESSAGE_INPUT_ID
  ) as HTMLInputElement;

  // Retrieve the AES key from local storage
  const aesKey = await deriveSharedSecret(keyPairB.privateKey, publicA);

  // Encrypt the message input value using the AES key
  const { encryptedData, iv } = await encrypt(aesKey, messageInput.value);

  // Update the encrypted URL input with the encrypted message
  const encryptedUrlInput = document.getElementById(
    ENCRYPTED_URL_INPUT_ID
  ) as HTMLInputElement;

  const ecdhPublicJwk = await window.crypto.subtle.exportKey(
    "jwk",
    keyPairB.publicKey
  );
  const url = new URL(window.location.toString());

  url.hash = `p=${encodeURIComponent(
    btoa(JSON.stringify(ecdhPublicJwk))
  )}&iv=${encodeURIComponent(
    btoa(String.fromCharCode.apply(null, Array.from(iv)))
  )}&m=${encodeURIComponent(
    btoa(
      String.fromCharCode.apply(null, Array.from(new Uint8Array(encryptedData)))
    )
  )}`;

  // url.hash = `p=${btoa(JSON.stringify(ecdhPublicJwk))}&iv=${btoa(
  //   String.fromCharCode.apply(null, Array.from(iv))
  // )}&m=${btoa(
  //   String.fromCharCode.apply(null, Array.from(new Uint8Array(encryptedData)))
  // )}`;

  encryptedUrlInput.value = url.toString();
}
